@@ -1,5 +1,5 @@
 /* This file is part of GNU cflow
-   Copyright (C) 1997,2005,2007 Sergey Poznyakoff
+   Copyright (C) 1997,2005,2007,2009 Sergey Poznyakoff
  
    GNU cflow is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,15 +18,20 @@
 
 #include <cflow.h>
 #include <argp.h>
+#include <argp-version-etc.h>
+#include <progname.h>
 #include <stdarg.h>
 #include <parser.h>
 #include <version-etc.h>
 
-const char *argp_program_version = "cflow (" PACKAGE_NAME ") " VERSION;
 const char *argp_program_bug_address = "<" PACKAGE_BUGREPORT ">";
 static char doc[] = N_("generate a program flowgraph")
 "\v"
 N_("* The effect of each option marked with an asterisk is reversed if the option's long name is prefixed with `no-'. For example, --no-cpp cancels --cpp.");
+const char *program_authors[] = {
+     "Sergey Poznyakoff",
+     NULL
+};
 
 enum option_code {
      OPT_DEFINES = 256,
@@ -213,7 +218,7 @@ int preprocess_option = 0; /* Do they want to preprocess sources? */
 
 char *start_name = "main"; /* Name of start symbol */
 
-Consptr arglist;        /* List of command line arguments */
+struct linked_list *arglist;        /* List of command line arguments */
 
 /* Given the option_type array and (possibly abbreviated) option argument
  * find the type corresponding to that argument.
@@ -257,14 +262,16 @@ symbol_override(const char *str)
      Symbol *sp;
      
      ptr = strchr(str, ':');
-     if (*ptr == ':') {
+     if (!ptr) {
+	  error(0, 0, _("%s: no symbol type supplied"), str);
+	  return;
+     } else {
 	  type = find_option_type(symbol_optype, ptr+1, 0);
 	  if (type == 0) {
 	       error(0, 0, _("unknown symbol type: %s"), ptr+1);
 	       return;
 	  }
-     } else
-	  type = IDENTIFIER;
+     } 
      name = strndup(str, ptr - str);
      sp = install(name);
      sp->type = SymToken;
@@ -480,7 +487,7 @@ set_level_indent(const char *str)
 static void
 add_name(const char *name)
 {
-     append_to_list(&arglist, (void*) name);
+     linked_list_append(&arglist, (void*) name);
 }
 
 static void
@@ -730,20 +737,13 @@ const char version_etc_copyright[] =
      year.  */
   "Copyright %s 2005, 2006, %d Sergey Poznyakoff";
 
-static void
-cflow_version(FILE *stream, struct argp_state *state)
-{
-     	version_etc(stream, "cflow", PACKAGE_NAME, PACKAGE_VERSION,
-		    "Sergey Poznyakoff", NULL);
-}
-
 int
 main(int argc, char **argv)
 {
      int index;
 
-     program_name = argv[0]; /* Until gnulib provides a better way */
-     argp_program_version_hook = cflow_version;
+     set_program_name(argv[0]);
+     argp_version_setup("cflow", program_authors);
      
      setlocale(LC_ALL, "");
      bindtextdomain(PACKAGE, LOCALEDIR);
@@ -769,16 +769,18 @@ main(int argc, char **argv)
 
      init();
 
-     if (arglist) 
-	  /* See comment to cleanup_processor */
-	  for (arglist = CAR(arglist); arglist; arglist = CDR(arglist)) {
-	       char *s = (char*)CAR(arglist);
+     if (arglist) {
+	  struct linked_list_entry *p;
+
+	  for (p = arglist->head; p; p = p->next) {
+	       char *s = (char*)p->data;
 	       if (s[0] == '-')
 		    pp_option(s);
 	       else if (source(s) == 0)
 		    yyparse();
 	  }
-
+     }
+     
      argc -= index;
      argv += index;
 
@@ -790,8 +792,6 @@ main(int argc, char **argv)
      if (input_file_count == 0)
 	     error(1, 0, _("no input files"));
 
-     cleanup();
-
      output();
      return 0;
 }
